//
//  MainController.m
//  RemovableColumns
//
//  Created by Andreas on 02.08.06.
//  Copyright 2006 Andreas Mayer. All rights reserved.
//

#import "MainController.h"
#import "AMRemovableColumnsTableView.h"
#import "DisplayOptionsWindowController.h"


@implementation MainController

- (void)awakeFromNib
{
	[tableView setObligatoryTableColumns:[NSSet setWithObject:[tableView tableColumnWithIdentifier:@"name"]]];
}

- (IBAction)viewOptions:(id)sender
{
	DisplayOptionsWindowController *controller = [[[DisplayOptionsWindowController alloc] init] autorelease];
	[controller setDelegate:self];
	[controller setIcon:[NSImage imageNamed:@"table"]];
	[controller setHeading:@"Test Table"];
	NSSet *allColumns = [tableView allTableColumns];
	NSSet *visibleColumns = [tableView visibleTableColumns];
	NSSet *obligatoryColumns = [tableView obligatoryTableColumns];
	NSMutableArray *options = [NSMutableArray array];
	NSMutableDictionary *option = nil;
	NSEnumerator *enumerator = [allColumns objectEnumerator];
	NSTableColumn *column;
	while (column = [enumerator nextObject]) {
		option = [NSMutableDictionary dictionaryWithCapacity:3];
		[option setObject:[column identifier] forKey:@"identifier"];
		[option setObject:[NSNumber numberWithBool:[visibleColumns containsObject:column]] forKey:@"selected"];
		[option setObject:[NSNumber numberWithBool:![obligatoryColumns containsObject:column]] forKey:@"enabled"];
		NSString *title = [[column headerCell] stringValue];
		[option setObject:title forKey:@"title"];
		[options addObject:option];
	}
	NSSortDescriptor *sortDescriptor = [[[NSSortDescriptor alloc] initWithKey:@"title" ascending:YES] autorelease];
	[controller setOptions:[options sortedArrayUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]]];
	[controller showWindow:self];
}

- (void)setViewOptions:(NSArray *)options
{
	NSTableColumn *column;
	NSEnumerator *enumerator = [options objectEnumerator];
	NSDictionary *option;
	while (option = [enumerator nextObject]) {
		column = [tableView tableColumnWithIdentifier:[option objectForKey:@"identifier"]];
		NSAssert2((column != nil), @"unknown column: %@ for table: %@", [option objectForKey:@"identifier"], [tableView autosaveName]);
		if ([[option objectForKey:@"selected"] boolValue]) {
			[(AMRemovableColumnsTableView *)tableView showTableColumn:column];
		} else {
			[(AMRemovableColumnsTableView *)tableView hideTableColumn:column];
		}
	}
}


- (IBAction)showReadMe:(id)sender
{
	// load text only once
	if ([[readMeTextView textStorage] length] == 0) {
		NSString *readMePath = [[NSBundle mainBundle] pathForResource:@"readme" ofType:@"rtf"];
		NSURL *readMeURL = [NSURL fileURLWithPath:readMePath];
		NSAttributedString *readMe = [[[NSAttributedString alloc] initWithURL:readMeURL documentAttributes:NULL] autorelease];
		[[readMeTextView textStorage] replaceCharactersInRange:NSMakeRange(0, [[readMeTextView textStorage] length]) withAttributedString:readMe];
		[readMeWindow center];
	}
	[readMeWindow makeKeyAndOrderFront:self];
}


@end
